@@ -2,22 +2,23 @@ const PREC = {
   COMMENT: -1,
 
   ASSIGN: 0,
-  TILDE: 1,
-  OR: 2,
-  AND: 3,
-  NOT: 4,
-  REL: 5,
-  PLUS: 6,
-  TIMES: 7,
-  SPECIAL: 8,
-  COLON: 9,
-  NEG: 10,
-  EXP: 11,
-  DOLLAR: 12,
-  NS_GET: 13,
-  CALL: 14,
-  SUBSET: 15,
-  FLOAT: 16
+  PIPE: 1,
+  TILDE: 2,
+  OR: 3,
+  AND: 4,
+  NOT: 5,
+  REL: 6,
+  PLUS: 7,
+  TIMES: 8,
+  SPECIAL: 9,
+  COLON: 10,
+  NEG: 11,
+  EXP: 12,
+  DOLLAR: 13,
+  NS_GET: 14,
+  CALL: 15,
+  SUBSET: 16,
+  FLOAT: 17
 }
 
 newline = '\n',
@@ -35,7 +36,8 @@ module.exports = grammar({
     program: $ => repeat(seq($._expression, optional(terminator))),
 
     _definition: $ => choice(
-      $.function_definition
+      $.function_definition,
+      $.lambda_function
       // TODO: other kinds of definitions
     ),
 
@@ -45,6 +47,12 @@ module.exports = grammar({
       $._expression
     )),
 
+    lambda_function: $ => prec.left(seq(
+      '\\',
+      $.formal_parameters,
+      $._expression
+    )),
+
     if: $ => prec.right(seq(
       'if',
       '(',
@@ -132,7 +140,8 @@ module.exports = grammar({
       $.left_assignment,
       $.left_assignment2,
       $.right_assignment,
-      $.super_assignment
+      $.super_assignment,
+      $.super_right_assignment,
     ),
 
     left_assignment: $ => prec.right(PREC.ASSIGN,
@@ -163,6 +172,13 @@ module.exports = grammar({
       field('value', $._expression)
     )),
 
+    super_right_assignment: $ => prec.right(PREC.ASSIGN,
+      seq(
+        field('value', $._expression),
+        '->>',
+        field('name', $._expression)
+      )),
+
     right_assignment: $ => prec.left(PREC.ASSIGN,
       seq(
         field('value', $._expression),
@@ -229,6 +245,12 @@ module.exports = grammar({
 
     dots: $ => '...',
 
+    pipe: $ => prec.left(PREC.PIPE, seq(
+      field('left', $._expression),
+      field('operator' ,'|>'),
+      field('right', $.call)
+    )),
+
     unary: $ => {
       const operators = [
         [PREC.NEG, choice('-', '+')],
@@ -250,7 +272,6 @@ module.exports = grammar({
         [prec.left, PREC.REL, choice('<', '>', '<=', '>=', '==', '!=')],
         [prec.left, PREC.OR, choice('||', '|')],
         [prec.left, PREC.AND, choice('&&', '&')],
-        [prec.left, PREC.SPECIAL, '|>'],
         [prec.left, PREC.SPECIAL, $.special],
         [prec.left, PREC.COLON, ':'],
         [prec.left, PREC.TILDE, '~'],
@@ -289,11 +310,13 @@ module.exports = grammar({
       $.string,
       $.call,
       $.function_definition,
+      $.lambda_function,
       $._assignment,
       $.brace_list,
       $.paren_list,
       $.binary,
       $.unary,
+      $.pipe,
       $.subset,
       $.subset2,
       $.dollar,
@@ -317,7 +340,7 @@ module.exports = grammar({
       ';'
     )),
 
-    identifier: $ => 
+    identifier: $ =>
       choice(
       /[A-Za-z.][A-Za-z0-9_.]*/,
       seq(
